@@ -15,8 +15,11 @@
 // Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 // 02110-1301, USA.
 
+#include <math.h>
+
 #include "point_type.h"
 
+#include "special_imptypes.h"
 #include "point_imp.h"
 #include "curve_imp.h"
 #include "line_imp.h"
@@ -405,7 +408,7 @@ void FixedPointType::executeAction(
     KigCommand* kc = new KigCommand( d, PointImp::stype()->moveAStatement() );
     mon.finish( kc );
 
-    d.history()->addCommand( kc );
+    d.history()->push( kc );
     break;
   };
   case 1:
@@ -444,7 +447,7 @@ void ConstrainedPointType::executeAction(
     po->setImp( new DoubleImp( newp ) );
     KigCommand* kc = new KigCommand( d, i18n( "Change Parameter of Constrained Point" ) );
     mon.finish( kc );
-    d.history()->addCommand( kc );
+    d.history()->push( kc );
     break;
   };
   default:
@@ -499,15 +502,9 @@ ObjectImp* MeasureTransportType::calc( const Args& parents, const KigDocument& d
 
   if ( parents.size() != 3 ) return new InvalidImp;
 
-  if ( parents[0]->inherits (SegmentImp::stype()) )
-  {
-    const SegmentImp* s = static_cast<const SegmentImp*>( parents[0] );
-    measure = s->length();
-  } else if ( parents[0]->inherits (ArcImp::stype()) )
-  {
-    const ArcImp* s = static_cast<const ArcImp*>( parents[0] );
-    measure = s->radius()*s->angle();
-  } else return new InvalidImp;
+  bool valid;
+  measure = getDoubleFromImp( parents[0], valid );
+  if ( ! valid ) return new InvalidImp;
 
   const Coordinate& p = static_cast<const PointImp*>( parents[2] )->coordinate();
   if ( parents[1]->inherits (LineImp::stype()) )
@@ -606,6 +603,9 @@ Args MeasureTransportType::sortArgs( const Args& args ) const
 }
 
 /* - transport of measure (old, for compatibility with prev. kig files) - */
+/* it should be safe now to remove this completely, since the loading of "MeasureTransport"
+ * is treated in "load07" building a node that uses the new version
+ */
 
 ObjectImp* MeasureTransportTypeOld::calc( const Args& parents, const KigDocument& doc ) const
 {
@@ -642,7 +642,7 @@ static const ArgsParser::spec argsspecMeasureTransportOld[] =
 KIG_INSTANTIATE_OBJECT_TYPE_INSTANCE( MeasureTransportTypeOld )
 
 MeasureTransportTypeOld::MeasureTransportTypeOld()
-  : ArgsParserObjectType( "MeasureTransport", argsspecMeasureTransportOld, 3 )
+  : ArgsParserObjectType( "MeasureTransportObsoleted", argsspecMeasureTransportOld, 3 )
 {
 }
 
@@ -663,3 +663,50 @@ const ObjectImpType* MeasureTransportTypeOld::resultId() const
 
 /* ----------------- end transport of measure ------------------------- */
 
+/* Construct a point whose coordinates are indicated by numeric labels    */
+
+ObjectImp* PointByCoordsType::calc( const Args& parents, const KigDocument& ) const
+{
+  if ( ! margsparser.checkArgs( parents ) ) return new InvalidImp;
+
+  bool valid;
+  double x = getDoubleFromImp( parents[0], valid);
+  if ( ! valid ) return new InvalidImp;
+  double y = getDoubleFromImp( parents[1], valid);
+  if ( ! valid ) return new InvalidImp;
+
+  const Coordinate nc = Coordinate( x, y );
+  if ( nc.valid() ) return new PointImp( nc );
+  else return new InvalidImp;
+}
+
+static const ArgsParser::spec argsspecPointByCoords[] =
+{
+  { &lengthimptypeinstance, "X coordinate given by this number/length",
+    I18N_NOOP( "Select a number/length as x coordinate of the point..." ), false },
+  { &lengthimptypeinstance, "Y coordinate given by this number/length",
+    I18N_NOOP( "Select a number/length as y coordinate of the point..." ), false }
+};
+
+KIG_INSTANTIATE_OBJECT_TYPE_INSTANCE( PointByCoordsType )
+
+PointByCoordsType::PointByCoordsType()
+  : ArgsParserObjectType( "PointByCoordinates", argsspecPointByCoords, 2 )
+{
+}
+
+PointByCoordsType::~PointByCoordsType()
+{
+}
+
+const PointByCoordsType* PointByCoordsType::instance()
+{
+  static const PointByCoordsType t;
+  return &t;
+}
+
+const ObjectImpType* PointByCoordsType::resultId() const
+{
+  return PointImp::stype();
+}
+
