@@ -1,5 +1,5 @@
 /* This file is part of KWordQuiz
-  Copyright (C) 2003 Peter Hedlund <peter.hedlund@kdemail.net>
+  Copyright (C) 2003-2008 Peter Hedlund <peter.hedlund@kdemail.net>
 
   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
@@ -19,25 +19,23 @@
 #ifndef QAVIEW_H
 #define QAVIEW_H
 
-#include <kaction.h>
+#include "ui_qaviewbase.h"
 
-#include "qaviewbase.h"
+#include <KActionCollection>
 
-#include "wqquiz.h"
-#include "wqscore.h"
+class KWQQuiz;
+class WQScore;
 
 /**
 @author Peter Hedlund
 */
-class QAView : public QAViewBase
+class QAView : public QWidget, public Ui::QAViewBase
 {
 Q_OBJECT
 public:
-    QAView(QWidget *parent = 0, const char *name = 0, WFlags f = 0);
+    QAView(QWidget *parent, KActionCollection * actionCollection);
 
-    ~QAView();
-
-    void setQuiz(WQQuiz *quiz);
+    void setQuiz(KWQQuiz *quiz);
     void init();
 
 public slots:
@@ -47,15 +45,11 @@ public slots:
     void slotRestart();
     void slotApplySettings();
     void slotSpecChar(const QChar &);
-    
-private:
-    WQQuiz *m_quiz;
-    WQScore *m_score;
 
-    KAction* actCheck;
-    KAction* actHint;
-    KAction* actRepeat;
-    KAction* actRestart;
+private:
+    KWQQuiz *m_quiz;
+    WQScore *m_score;
+    KActionCollection *m_actionCollection;
 
     int m_question;
     int m_error;
