@@ -155,7 +155,7 @@ void MarblePart::exportMapScreenShot()
         // Take the case into account where no file format is indicated
         char * format = 0;
         if ( !fileName.endsWith("png", Qt::CaseInsensitive) 
-           | !fileName.endsWith("jpg", Qt::CaseInsensitive) )
+           && !fileName.endsWith("jpg", Qt::CaseInsensitive) )
         {
             format = (char*)"JPG";
         }
